slots_wide = 3;
slots_tall = 3;

module lid(x, y, z) {
    z2 = 2 * z / (2 + sqrt(2));
    intersection() {
        translate([0, 0, 0]) difference() {
            minkowski() {
                $fn=4;
                translate([0, 0, 0]) cube([x - z2 * 2, y, z]);
                union() {
                    translate([z2, z - z2, 0]) cylinder(r1=0, r2=z2, h=z2);
                }
            }
            minkowski() {
                $fn=8;
                translate([z / 2 + inset_distance, z, 0]) cube([x - z - inset_distance * 2, y - z / 2 - inset_distance, z - inset_distance]);
                translate([0, 0, z / 2]) sphere(r=z / 2 - inset_distance);
            }
        }
        translate([0, z, 0]) {
            cube([x, y, z - distance / 2]);
        }
    }
}

module slot_latch() {
    $fn=4;
    cube([slot_latch_height, latch_depth, slot_depth]);
    intersection() {
        translate([0, latch_radius * 2 / 3, slot_depth]) rotate([0, 90, 0]) {
            scale([2 / 3, 2 / 3, 1]) cylinder(h=slot_latch_height, r=latch_radius);
            translate([slot_depth + latch_depth + latch_radius, -latch_radius * 2 / 3]) cylinder(h=slot_latch_height, r=(slot_depth + latch_depth + latch_radius));
        }
        translate([0, -slot_depth - latch_depth, 0]) cube([slot_latch_height, slot_depth + latch_depth * 2, slot_depth + latch_depth + latch_radius]);
    }
}

module slot() {
    %translate([slot_radius, latch_depth + slot_radius, 0]) minkowski() {
        cube([slot_height - slot_radius * 2, slot_width - slot_radius * 2, slot_depth / 2]);
        cylinder(r=slot_radius, h=slot_depth / 2);
    }
    translate([slot_radius, latch_depth + slot_radius, 0]) difference() {
        minkowski() {
            translate([0, 0, -0.1]) cube([slot_height - slot_radius * 2, slot_width - slot_radius * 2, 0.1]);
            cylinder(r1=slot_radius * 2, r2=slot_radius, h=0.25);
        }
        minkowski() {
            translate([0, 0, -0.1]) cube([slot_height - slot_radius * 2, slot_width - slot_radius * 2, 0.1]);
            cylinder(r=slot_radius, h=0.3);
        }
        translate([slot_height / 2 - slot_radius, -slot_radius / 2, 0]) cylinder(r1=0, r2=slot_radius * 2, h=0.25);
        translate([slot_height / 2 - slot_radius, slot_width - slot_radius * 1.5, 0]) cylinder(r1=0, r2=slot_radius * 2, h=0.25);
    }

    translate([slot_radius, 0, 0]) slot_latch();
    translate([slot_radius, slot_width + latch_depth * 2, 0]) mirror([0, 1, 0]) slot_latch();
    translate([slot_height - slot_latch_height - slot_radius, 0, 0]) slot_latch();
    translate([slot_height - slot_latch_height - slot_radius, slot_width + latch_depth * 2, 0]) mirror([0, 1, 0]) slot_latch();
}

module side(x, y, z) {
    lid(x, y, z);
    $fn=$fn / 2;
    translate([0, 0, inset_distance]) {
        for (j = [0:slots_tall - 1]) {
            for (i = [0:slots_wide - 1]) {
                translate([0.75 + (slot_height + 0.5) * j, 1.9 + (slot_width + 0.8) * i, 0]) {
                    slot();   
                }
            }
        }
    }
}

module cap(r) {
    difference() {
        intersection() {
            translate([r, r, r]) scale([1, 1 / cos(22.5), 1 / cos(22.5)]) rotate([0, 90, 0]) rotate([0, 0, 22.5]) sphere(r / cos(22.5), $fn=8);
            cube([r, r * 2, r * 2]);
        }
    }
}

module capcut(r) {
    translate([r, r, r]) sphere(cap_radius);
}

module latch(d) {
    d2 = d / 2;
    $fn = $fn * 0.5;
    translate([-d - latch_width / 2, -d2 - latch_depth, 0]) {
        cube([latch_width + d * 2, latch_depth + d2, latch_radius + latch_height]);
    }
    translate([-d - latch_width / 2, -latch_height / 2 - latch_depth, 0]) {
        difference() {
            cube([latch_width + d * 2, latch_height / 2, latch_height - d2]);
            translate([0, 0, latch_height - d2]) {
                scale([1, 0.5, 1]) rotate([0, 90, 0]) cylinder(h=latch_width + d * 2, r=latch_height - d2);
            }
        }
    }
    translate([-d - latch_width / 2, -latch_radius + latch_outset, latch_radius + latch_height + d2 / 2]) {
        rotate([0, 90, 0]) cylinder(h=latch_width + d * 2, r=latch_radius + d2);
    }
}

module opposing(x) {
    children();
    translate([x, 0, 0]) mirror([1, 0, 0]) children();
}

module top(x, y, z, d) {
    cut = x - z * 2;
    difference() {
        union() {
            side(x, y, z);
            translate([0, 0, 0]) opposing(x) cap(z);
            translate([z + hinge_size + d * 2, z, z]) {
                rotate([22.5, 0, 0]) rotate([0, 90, 0]) cylinder(r=z / cos(22.5), h=cut - (hinge_size + d * 2) * 2, $fn=8);
                translate([0, 0, inset_distance - z]) {
                    cube([cut - (hinge_size + d * 2) * 2, z, z]);
                    intersection() {
                        rotate([-45, 0, 0]) cube([cut - (hinge_size + d * 2) * 2, z - d, z]);
                        cube([cut - (hinge_size + d * 2) * 2, z * 2, z]);
                    }
                }
            }
            opposing(x) {
                difference() {
                    translate([inset_distance, z, inset_distance]) {
                        intersection() {
                            translate([0, 0, -d]) rotate([-45, 0, 0]) cube([z - inset_distance, z - d, z]);
                            cube([z - inset_distance, z * 2, z]);
                        }
                        cube([z - inset_distance, z, z - inset_distance - d / 2]);
                    }
                    translate([0, z * 3 + d, 0]) rotate([90, 0, 0]) cylinder(r=2 * z / (2 + sqrt(2)), h=z * 2 + d * 2, $fn=4);
                }
            }
        }
        translate([d, d, d]) opposing(x - d * 2) capcut(z - d);
        translate([z, z, z]) {
            rotate([0, 90, 0]) {
                cylinder(r=cap_radius, h=cut);
            }
        }
        opposing(x) {
            translate([z, z / 2 / sqrt(2), d / 2]) {
                rotate([-45, 0, 0]) cube([hinge_size + d * 2, z * cos(22.5), z * 1.75]);
            }
        }
        translate([(x + latch_width + latch_distance) / 2, y + z, z - d / 2]) {
            scale([1, 1, -1]) latch(latch_distance);
        }
    }
    translate([(x - latch_width - latch_distance) / 2, y + z, z - d / 2]) {
        latch(0);
    }
}

module bottom(x, y, z, d) {
    cap_size = cap_radius - d;
    difference() {
        side(x, y, z);
        translate([0, z / 2 / sqrt(2), -d / 2]) {
            rotate([-45, 0, 0]) cube([x, z * cos(22.5), z * 1.82]);
        }
        translate([0, 0.1899, 0.3807]) cube([x, z * 2, z]);
        cube([x, z + d * 2, z]);
        translate([(x - latch_width - latch_distance) / 2, y + z, z - d / 2]) {
            scale([1, 1, -1]) latch(latch_distance);
        }
    }
    translate([z, z, z]) {
        rotate([0, 90, 0]) {
            cylinder(r=cap_size, h=x - z * 2);
        }
    }
    opposing(x) {
        translate([z, z, z]) {
            sphere(cap_size);
        }
        translate([z + d, z, z]) {
            rotate([0, 90, 0]) {
                rotate([0, 0, 22.5]) cylinder(r=z / cos(22.5), h=hinge_size, $fn=8);
            }
        }
        translate([z + d, z, 0]) {
            intersection() {
                translate([0, 0, inset_distance - d]) rotate([-45, 0, 0]) cube([hinge_size, z - d, z]);
                cube([hinge_size, z * 2, z]);
            }
            cube([hinge_size, z, z]);
        }
    }

    translate([(x + latch_width + latch_distance) / 2, y + z, z - d / 2]) {
        latch(0);
    }
}

$fn = 60;

curve = 0.65;
distance = 0.06;
inset_distance = 0.12;

cap_radius = 0.55;

latch_width = 1;
latch_depth = 0.125;
latch_radius = 0.118;
latch_height = 0.125;
latch_outset = 0.06;
latch_distance = 0.075;

/*slot_width = 3.35;
slot_height = 3.55;
slot_depth = 0.4;*/
slot_width = 2.15;
slot_height = 3.15 + 0.05;
slot_depth = 0.4;
slot_radius = 0.3;
slot_latch_height = 0.5;

length = (slot_width + 0.8) * slots_wide + 1.4;
width = (slot_height + 0.5) * slots_tall + 1.0;

hinge_size = (width - curve * 2 - distance * 4) / 3;

scale(10) {
    translate([0, -curve, 0]) {
        top(width, length, curve, distance);
    }

    translate([0, curve, 0]) {
        scale([1, -1, 1]) {
            bottom(width, length, curve, distance);
        }
    }
}